--------------------------------------------------------------------------------
-- Title   : frequency division block
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : frequency division for DG slicing
--               Fout = 1000.7558192399 Hz = CLK_SR / 846 = slicing laser freq
--------------------------------------------------------------------------------
-- File     : freqDIV_clkdiv.vhd
-- Revision : x.x.x
-- Created  : February 03, 2014
-- Updated  : February 03, 2014
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2014 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity freqDIV_clkdiv is
  port (
    p_clkSR   : in  std_logic;                     -- clk input
    p_reset   : in  std_logic;                     -- reset input
    p_clkOUT  : out std_logic_vector (4 downto 0); -- clk outputs
    p_outPECL : out std_logic_vector (4 downto 0)  -- LVPECL outputs
  );
end freqDIV_clkdiv;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_freqDIV_clkdiv of freqDIV_clkdiv is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------
  constant c_lfsr   : std_logic_vector (8 downto 0) := "111100011";

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_lfsr     : std_logic_vector (8 downto 0);
  signal s_lfsr_lsb : std_logic;
  signal s_div_423  : std_logic;
  signal s_div_846  : std_logic;

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- s_div_423 = 2001.5116384798 Hz = CLK_SR / 423
  process (p_clkSR, p_reset)
  begin
    if (p_reset = '1') then
      s_lfsr <= (OTHERS => '0');
      s_div_423 <= '0';
    elsif (rising_edge(p_clkSR)) then
      if(s_lfsr = c_lfsr) then
        s_lfsr <= (OTHERS => '0');
        s_div_423 <= '1';
      else
        s_lfsr <= s_lfsr(7 downto 0) & s_lfsr_lsb;
        s_div_423 <= '0';
      end if;
    end if;
  end process;

  -- s_div_846 = s_div_423 / 2 = 1000.7558192399 Hz = CLK_SR / 846
  process (p_reset, s_div_423)
  begin
    if (p_reset = '1') then
      s_div_846 <= '0';
    elsif (rising_edge(s_div_423)) then
      s_div_846 <= not s_div_846;
    end if;
  end process;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  s_lfsr_lsb  <= s_lfsr(8) xnor s_lfsr(4);

  p_clkOUT(1) <= s_div_846;
  p_clkOUT(2) <= s_div_846;
  p_clkOUT(3) <= s_div_846;
  p_clkOUT(4) <= s_div_846;

  -- Unused signals
  p_clkOUT(0) <= '0'; -- the IO is used as input
  p_outPECL   <= "00000";

end rtl_freqDIV_clkdiv;
